aboutsummaryrefslogtreecommitdiffstats
path: root/activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb
diff options
context:
space:
mode:
authorJeremy Kemper <jeremy@bitsweat.net>2009-01-10 13:36:09 -0800
committerJeremy Kemper <jeremy@bitsweat.net>2009-01-10 13:39:37 -0800
commitab0ce052ba23a4cce7a84ecade0d00d9cc518ebd (patch)
tree431f973e36b90deaf1c78aaa7a0ee139a4336469 /activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb
parent223a1d9451c88800e9fcc93a726fdebec99e2650 (diff)
downloadrails-ab0ce052ba23a4cce7a84ecade0d00d9cc518ebd.tar.gz
rails-ab0ce052ba23a4cce7a84ecade0d00d9cc518ebd.tar.bz2
rails-ab0ce052ba23a4cce7a84ecade0d00d9cc518ebd.zip
Introduce transaction_joinable flag to mark that the fixtures transaction can't joined, a new savepoint is required even if :requires_new is not set. Use :requires_new option instead of :nest. Update changelog.
[#383 state:committed]
Diffstat (limited to 'activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb')
-rw-r--r--activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb31
1 files changed, 16 insertions, 15 deletions
diff --git a/activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb b/activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb
index cecbc6b3ac..39118583bd 100644
--- a/activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb
+++ b/activerecord/lib/active_record/connection_adapters/abstract/database_statements.rb
@@ -89,14 +89,8 @@ module ActiveRecord
# - The block will be run without doing anything. All database statements
# that happen within the block are effectively appended to the already
# open database transaction.
- # - However, if +start_db_transaction+ is set to true, then the block will
- # be run inside a new database savepoint, effectively making the block
- # a sub-transaction.
- # - If the #transactional_fixtures attribute is set to true, then the first
- # nested call to #transaction will create a new savepoint instead of
- # doing nothing. This makes it possible for toplevel transactions in unit
- # tests to behave like real transactions, even though a database
- # transaction has already been opened.
+ # - However, if +requires_new+ is set, the block will be wrapped in a
+ # database savepoint acting as a sub-transaction.
#
# === Caveats
#
@@ -111,20 +105,25 @@ module ActiveRecord
# already-automatically-released savepoints:
#
# Model.connection.transaction do # BEGIN
# Model.connection.transaction(true) do # CREATE SAVEPOINT rails_savepoint_1
# Model.connection.create_table(...)
- # # rails_savepoint_1 now automatically released
- # end # RELEASE savepoint rails_savepoint_1 <--- BOOM! database error!
+ # # active_record_1 now automatically released
+ # end # RELEASE SAVEPOINT active_record_1 <--- BOOM! database error!
# end
- def transaction(start_db_transaction = false)
- start_db_transaction ||= open_transactions == 0 || (open_transactions == 1 && transactional_fixtures)
+ def transaction(options = {})
+ options.assert_valid_keys :requires_new, :joinable
+
+ last_transaction_joinable, @transaction_joinable =
+ @transaction_joinable, options[:joinable] || true
+ requires_new = options[:requires_new] || !last_transaction_joinable
+
transaction_open = false
begin
if block_given?
- if start_db_transaction
+ if requires_new || open_transactions == 0
if open_transactions == 0
begin_db_transaction
- else
+ elsif requires_new
create_savepoint
end
increment_open_transactions
@@ -145,6 +144,8 @@ module ActiveRecord
raise unless database_transaction_rollback.is_a? ActiveRecord::Rollback
end
ensure
+ @transaction_joinable = last_transaction_joinable
+
if outside_transaction?
@open_transactions = 0
elsif transaction_open
@@ -111,20 +105,25 @@ module ActiveRecord
#
# Model.connection.transaction do # BEGIN
- # Model.connection.transaction(true) do # CREATE SAVEPOINT rails_savepoint_1
+ # Model.connection.transaction(:requires_new => true) do # CREATE SAVEPOINT active_record_1
# Model.connection.create_table(...)